import React, { useMemo, useRef } from 'react'
import { Table, Tooltip, Button, Popconfirm, Input, Space } from 'antd'
import { SearchOutlined } from '@ant-design/icons'
import FileView from './components/FileView'
import CoordView from './components/CoordView'
import SelectView from './components/SelectView'
import IconPack from '../../../components/IconPack'
import { isObject } from '../../../../utils'

const getFileInfo = (formJson) => {
  let obj = {}
  let parent = formJson?.properties
  if (isObject(parent)) {
    for (let v in parent) {
      let child = parent[v]?.properties
      if (isObject(child)) {
        for (let s in child) {
          obj[s] = child[s]
        }
      }
    }
  }
  return obj
}

const TablePack = (props) => {

  const { parent, readOnly, config, loading, dataSource, rowSelection, onRow, tableChange, btnsClick } = props
  const searchInput = useRef(null)

  const handleReset = (clearFilters, confirm) => {
    clearFilters()
    confirm({ closeDropdown: true })
  }

  const getColumnSearchProps = (dataIndex) => ({
    filterDropdown: ({ setSelectedKeys, selectedKeys, confirm, clearFilters, close }) => (
      <div
        style={{ padding: 8 }}
        onKeyDown={(e) => e.stopPropagation()}
      >
        <Input
          ref={searchInput}
          placeholder={`搜索 ${dataIndex}`}
          value={selectedKeys[0]}
          onChange={e => setSelectedKeys(e.target.value ? [e.target.value] : [])}
          onPressEnter={() => confirm({ closeDropdown: true })}
          style={{
            marginBottom: 8,
            display: 'block',
            width: '280px',
          }}
        />
        <Space>
          <Button
            type="primary"
            onClick={() => confirm({ closeDropdown: true })}
            icon={<SearchOutlined />}
            size="small"
            style={{ width: 90 }}
          >
            搜索
          </Button>
          <Button
            onClick={() => handleReset(clearFilters, confirm)}
            size="small"
            style={{ width: 90 }}
          >
            重置
          </Button>
        </Space>
      </div>
    ),
    filterIcon: (filtered) => (
      <SearchOutlined
        style={{
          color: filtered ? '#1890ff' : undefined,
        }}
      />
    ),
  })

  const btns = useMemo(() => {
    if (readOnly) {
      return [
        {
          title: '详情',
          icon: <IconPack.XiangQing />,
        },
      ]
    }
    return [
      {
        title: '详情',
        icon: <IconPack.XiangQing />,
      },
      {
        title: '编辑',
        icon: <IconPack.BianJi />,
      },
      {
        title: '删除',
        icon: <IconPack.ShanChu />,
      }
    ]
  }, [readOnly, parent])

  const fileColumns = useMemo(() => {
    const { webShowFieldGroup, formJson } = config
    let showField = webShowFieldGroup ? webShowFieldGroup.split(',').filter(v => v) : []
    let json = getFileInfo(formJson)
    let array = []
    showField.forEach(v => {
      array.push({
        title: json[v]?.title || v,
        dataIndex: v,
        width: 120,
        ellipsis: true,
        sorter: true,
        filterSearch: true,
        ...getColumnSearchProps(v),
        render: (value, r) => {
          let props = { ...json[v], value }
          const { widget } = props
          if (widget === 'FileUpload') {
            return <FileView {...props} />
          }
          if (['Coordinate', 'Device', 'DrawPath', 'DrawArea'].includes(widget)) {
            return <CoordView {...props} />
          }
          if (['RadioButton', 'CheckBox'].includes(widget)) {
            return <SelectView {...props} />
          }
          return (
            <div>{value}</div>
          )
        }
      })
    })
    return array
  }, [config])

  const columns = useMemo(() => {
    let columns = [
      {
        title: '序号',
        dataIndex: 'r',
        key: 'r',
        width: 50,
        align: 'center',
        fixed: 'left',
      },
      ...fileColumns,
      {
        title: '操作',
        width: btns.length * 35 + 30,
        align: 'center',
        fixed: 'right',
        render: (r) => {
          return (
            <div style={{ display: 'flex', justifyContent: 'center' }}>
              {
                btns.map(v => {
                  if (v.title === '删除') {
                    return (
                      <Popconfirm
                        key={v.title}
                        title='确定要删除数据吗？'
                        onConfirm={() => btnsClick(v.title, r.ID)}
                        okText='确定'
                        cancelText='取消'
                      >
                        <Button
                          type='link'
                          title='删除'
                          style={{ padding: '0 7px' }}
                          icon={v.icon}
                        />
                      </Popconfirm>
                    )
                  }
                  return (
                    <Button
                      key={v.title}
                      type='link'
                      title={v.title}
                      icon={v.icon}
                      style={{ padding: '0 7px' }}
                      onClick={() => btnsClick(v.title, r.ID)}
                    >
                    </Button>
                  )
                })
              }
            </div>
          )
        }
      }
    ]
    if (parent === '台账选择器') {
      return fileColumns
    }
    return columns
  }, [parent, btns, fileColumns])

  return (
    <Table
      size='small'
      rowKey='ID'
      bordered
      loading={loading}
      columns={columns}
      dataSource={dataSource}
      rowSelection={rowSelection}
      onRow={onRow}
      pagination={false}
      scroll={{ y: 'calc(100% - 40px)' }}
      onChange={tableChange}
    />
  )

}

export default TablePack